/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-10-17 15:18:02.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.common.jpa;

import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.data.jpa.domain.Specifications;

import javax.persistence.criteria.Path;
import javax.persistence.criteria.Predicate;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;

/**
 * JPA Spec常用的查询封装类
 *
 * @author: fengdesheng
 * @since: 2018/10/17
 */
public class SpecUtils {
    /**
     * 默认当前页数
     */
    public static final int DEFAULT_PAGE_NO = 1;
    /**
     * 默认每页条数
     */
    public static final int DEFAULT_PAGE_SIZE = 10;

    /**
     * 构建分页条件
     *
     * @param pageNo
     * @param pageSize
     * @return
     */
    public static Pageable buildPageable(Integer pageNo, Integer pageSize) {
        return buildPageable(pageNo, pageSize, null);
    }

    /**
     * 构建分页条件
     *
     * @param pageNo
     * @param pageSize
     * @param sort
     * @return
     */
    public static Pageable buildPageable(Integer pageNo, Integer pageSize, Sort sort) {
        if (pageNo == null || pageNo < 1) {
            pageNo = DEFAULT_PAGE_NO;
        }
        if (pageSize == null || pageSize < 1) {
            pageSize = DEFAULT_PAGE_SIZE;
        }
        return new PageRequest(pageNo - 1, pageSize, sort);
    }

    /**
     * 构建查询条件
     *
     * @param clazz
     * @param searchers
     * @param <T>
     * @return
     */
    public static <T> Specification<T> buildSearchers(final Class<T> clazz, final List<SearchBuilder.Searcher> searchers) {
        return buildSearchers(clazz, searchers, false);
    }

    /**
     * 构建查询条件
     *
     * @param clazz
     * @param searchers
     * @param isOR
     * @param <T>
     * @return
     */
    public static <T> Specification<T> buildSearchers(final Class<T> clazz, final List<SearchBuilder.Searcher> searchers, final boolean isOR) {
        if (searchers == null || searchers.size() == 0) {
            return null;
        }
        return (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            for (SearchBuilder.Searcher filter : searchers) {
                if (filter.getFieldName() == null || "".equals(filter.getFieldName())) {
                    continue;
                }
                String[] names = filter.getFieldName().split("\\.");
                Path expression = root.get(names[0]);
                for (int i = 1; i < names.length; i++) {
                    expression = expression.get(names[i]);
                }
                switch (filter.getOp()) {
                    case EQ:
                        predicates.add(builder.equal(expression, filter.getValue()));
                        break;
                    case NOT_EQ:
                        predicates.add(builder.notEqual(expression, filter.getValue()));
                        break;
                    case GT:
                        predicates.add(builder.greaterThan(expression, (Comparable) filter.getValue()));
                        break;
                    case GTE:
                        predicates.add(builder.greaterThanOrEqualTo(expression, (Comparable) filter.getValue()));
                        break;
                    case LT:
                        predicates.add(builder.lessThan(expression, (Comparable) filter.getValue()));
                        break;
                    case LTE:
                        predicates.add(builder.lessThanOrEqualTo(expression, (Comparable) filter.getValue()));
                        break;
                    case LIKE:
                        predicates.add(builder.like(expression, "%" + filter.getValue() + "%"));
                        break;
                    case L_LIKE:
                        predicates.add(builder.like(expression, "%" + filter.getValue()));
                        break;
                    case R_LIKE:
                        predicates.add(builder.like(expression, filter.getValue() + "%"));
                        break;
                    case IS_NULL:
                        predicates.add(builder.isNull(expression));
                        break;
                    case NOT_NULL:
                        predicates.add(builder.isNotNull(expression));
                        break;
                    case IN:
                        predicates.add(expression.in((Object[]) filter.getValue()));
                        break;
                    case NOT_IN:
                        predicates.add(expression.in((Object[]) filter.getValue()).not());
                        break;
                }
            }
            if (!predicates.isEmpty()) {
                if (isOR) {
                    //将所有条件用 or 联合起来
                    return builder.or(predicates.toArray(new Predicate[predicates.size()]));
                } else {
                    //将所有条件用 and 联合起来
                    return builder.and(predicates.toArray(new Predicate[predicates.size()]));
                }
            }
            return builder.conjunction();
        };
    }

    /**
     * 合并查询条件
     *
     * @param specs
     * @return
     */
    public static Specification andMerge(@NotNull Specification... specs) {
        if (specs == null || specs.length == 0) {
            return null;
        }
        Specifications result = null;
        for (int i = 0; i < specs.length; i++) {
            if (i == 0) {
                result = Specifications.where(specs[i]);
                continue;
            }
            result = Specifications.where(result).and(specs[i]);
        }
        return result;
    }

    /**
     * 合并查询条件
     * 注：SQL中AND优先级高于OR
     * 示例：... t.name='' or t.age=1 and t.gender=1 等同于 ... t.name='' or (t.age=1 and t.gender=1)
     *
     * @param specs
     * @return
     */
    public static Specification orMerge(@NotNull Specification... specs) {
        if (specs == null || specs.length == 0) {
            return null;
        }
        Specifications result = null;
        for (int i = 0; i < specs.length; i++) {
            if (i == 0) {
                result = Specifications.where(specs[i]);
                continue;
            }
            result = Specifications.where(result).or(specs[i]);
        }
        return result;
    }

}